//Source file for Rendering ASCII art sprites.

#include <header.h>
#include <global.h>
#include <gameglobal.h>
#include <ascrender.h>
#include <log.h>

//ASCII Render Buffer
#define ASCBUFSIZE 128 //Actually, not more than (xres/xmul)+1 is needed :)

//Blit ASCII: Simplified version of Allegro Blit
void asciiart_blit(aasprite *src, aasprite *dest, int sx, int sy, int dx, int dy, int w, int h){
      int i;	      
      size_t wbytes=0;	
      unsigned char *srcptrpt,*destptrpt;
	
      if(sx>src->width||sy>src->height) return;
      if(dx>dest->width||dy>dest->height) return;
	
      if(sx<0) { w+=sx; dx-=sx; sx=0; }
      if(sy<0) { h+=sy; dy-=sy; dy=0; }
 
      if(sx+w>src->width) w=src->width-sx;
      if(sy+h>src->height) h=src->height-sy;

      if(dx<0) { w+=dx; sx-=dx; dx=0; }
      if(dy<0) { h+=dy; sy-=dy; dy=0; }
	
      if(dx+w>dest->width) w=dest->width-dx; 
      if(dx+h>dest->height) h=dest->height-dy;
       
      if(w<=0||h<=0) return; 

      wbytes=w*sizeof(unsigned char);
      srcptrpt=src->data+(sy*src->width)+sx;
      destptrpt=dest->data+(dy*dest->width)+sy;
      for(i=0;i<h;++i){
	      memcpy(destptrpt,srcptrpt,wbytes);  	
	      srcptrpt+=src->width;
	      destptrpt+=dest->width;
      } 
	
      return;
	
}

//Clear ASCII to Value
void asciiart_clear_to_value(aasprite *givaas, unsigned char value){
	memset(givaas->data,value,sizeof(unsigned char)*givaas->width*givaas->height); //Assume
}

//Clear ASCII
void asciiart_clear(aasprite *givaas){
	asciiart_clear_to_value(givaas,AART_NULL);
}

//Render ASCII Art
void asciiart_bitmap_blit(aasprite *givaas, BITMAP *bitmap, int sx, int sy, int ex, int ey, int x, int y){
        int w,h;
	int wb,hb;
	int i;
	int color=0;
	size_t wbytes=0;
        unsigned char *srcptrpt;
	unsigned char ascrndbuffer[ASCBUFSIZE];

	//Cut Here
        if(sx>=givaas->width) return;
        if(sy>=givaas->height) return;

        if(sx<0) sx=0;
        if(sy<0) sy=0;

        if(ex>=givaas->width) ex=givaas->width-1;
        if(ey>=givaas->height) ey=givaas->height-1;

	if(x<0) { sx+=(-x)/xmul; x=-((-x)%xmul); }
	if(y<0) { sy+=(-y)/ymul; y=-((-y)%ymul); } 

	//Width/Height	
	w=ex-sx+1;
	h=ey-sy+1;

	//Cut Here
        if(w<=0||h<=0) return;

        wb=w*xmul;
        hb=h*ymul;
	
	if(x+wb>bitmap->w) wb=bitmap->w-x;
	if(y+hb>bitmap->h) hb=bitmap->h-y;

	//Cut Here
        if(wb<=0||hb<=0) return;

	//Use for rendering
	w=wb/xmul; if(wb%xmul) ++w;	
	h=hb/ymul; if(hb%ymul) ++h;

	//Precalculate
        wbytes=w*sizeof(unsigned char);

        //For printf
        *(ascrndbuffer+w)=0;

	color=makecol(givaas->r,givaas->g,givaas->b);
        srcptrpt=givaas->data+(sy*givaas->width)+sx;
	for(i=0;i<h;++i){
	      memcpy(ascrndbuffer,srcptrpt,wbytes);
	      textout_ex(bitmap,font,ascrndbuffer,x,y,color,-1);
	      srcptrpt+=givaas->width;
              y+=ymul;
	}
}

//Draw ASCII Art
void draw_asciiart(BITMAP *bitmap, aasprite *givaas, int x, int y){
	asciiart_bitmap_blit(givaas,bitmap,0,0,givaas->width-1,givaas->height-1,x,y);
}
