//Source file for Game Play.

#include <camera.h>
#include <train.h>
#include <player.h>
#include <level.h>
#include <levrender.h>
#include <global.h>
#include <gamestate.h>
#include <gameglobal.h>
#include <gamerender.h>
#include <gameload.h>
#include <log.h>

int flags=0;
int ingamehelp=0;

//Play Game Music
void game_stop_music(){
	if(!soundsystem) return;
	stop_midi();
}

//Stop Game Music
void game_play_music(){
	if(!soundsystem) return;
	
	play_midi(trainmusic, TRUE);
}

//Current Level Init
void pre_cur_level_quit(){
	//UnInitialize Music
	game_stop_music();

	//Uninitialize Player and Trains
	player_uninitialize(curlevel);
	trains_uninitialize(curlevel);
	
	//Misc Data
	levelwidth=0;levelheight=0;
	levelwon=0;
	gamewon=0; gamelost=0;
	gametimer=0; availtime=0;
	am=NULL;
}

//Level PreInitialize
void pre_level_init(int levelnum){
	am=levels[levelnum]->am;

	levelwidth=am->width*am->tilewidth; levelheight=am->height*am->tileheight;
	
	//Initialize Timers
	availtime=lps*am->timesec;
	gametimer=0;

	//Initialize Win/Lose
	gamewon=0; gamelost=0;
	levelwon=0;

	//Initialize Camera
	camera_init();

	//Initialize Trains and Player
	trains_initialize(levelnum);
	player_initialize(levelnum);

	//Initialize Music
	game_play_music();
}


//Current Level Init
void pre_cur_level_init(){
	pre_level_init(curlevel);
}

//First Level
void first_level(){
	curlevel=0;
	pre_cur_level_init();
	log_printf("First Level");
}

//Next Level
void next_level(){
	pre_cur_level_quit();
	curlevel++; 
	if(curlevel>nlevels) curlevel=0; 
	pre_cur_level_init();
	log_printf("Next Level, @Level %d",curlevel);
}

//Draw Level
void level_draw(int levelnum){
	render_level(dbuffer, levels[levelnum], camx, camy, flags);
}

//Current Level Draw
void cur_level_draw(){
	level_draw(curlevel);
}

//Draw Game Stuff
void cur_gamestuff_draw(){
	render_gamestuff(dbuffer,camx,camy,flags);
}

//Start Game
void game_open(){
	first_level();
}

//End Game
void game_close(){
	pre_cur_level_quit();
	state_switch(GAME_MENU);
}


//Win Game
void game_win(){
	if(gamewon) {
		pre_cur_level_quit();
		state_switch(GAME_WIN);
	}
}

//Lose Game
void game_lose(){
	if(gamelost) {
		pre_cur_level_quit();
		state_switch(GAME_LOST);
	}
}

//Win Level
void level_win(){
	if(levelwon) {
		if(curlevel==(nlevels-1)) ++gamewon;
		else next_level();
	}
}

//Game Play Init
void game_play_init(){
	int status=-1;

	//Load Levels
	status=load_levels();
	
	//Load Sprites				
	if(!status)
		status=load_game_sprites();

	//Load Sounds	
	if(soundsystem&&!status)
		status=load_game_sounds();
				
	//Initialize Level Renderer
	if(!status)
		status=init_level_renderer(); 

	//Initialize Level Renderer
	if(!status)
		status=init_game_renderer(); 

	if(status) //Quit
		state_switch(GAME_MENU);

	else	//Start Game
		game_open();
}

//Game Play Logic
void game_play_logic(){
	if(ingamehelp) return;

	++gametimer;
	--availtime;

	update_player();
	update_trains();

	if(availtime<=0)
		++gamelost;

	++camx; right_camera();

	game_lose();
	level_win();
	game_win();
}

//Game Play Input
void game_play_input(){
	int k=-1;

	if(ingamehelp) {
		if(!keypressed()) return;
	
		k=readkey(); k>>=8;

		if(k==KEY_ENTER) ingamehelp=0;
		return;
	}
	if(key[KEY_F1])	{ ingamehelp++; return; }

	if(key[KEY_ESC]) {
		game_close(); return;
	}
	if(key[KEY_PGUP]) up_camera();
	if(key[KEY_PGDN]) down_camera();
	
	if(key[KEY_C]) player_crouch();
	if(key[KEY_SPACE]) player_jump();
	if(key[KEY_RIGHT]) player_right();
}

//Game Play Render
void game_play_render(){
	if(ingamehelp) {
		game_help_render(); 
		return;
	}
	cur_level_draw();
	cur_gamestuff_draw();
}

//Game Play Quit
void game_play_quit(){
	//Quit Game Renderer	
	quit_game_renderer();

	//Quit Level Renderer	
	quit_level_renderer();

	//Unload Sounds	
	if(soundsystem) unload_game_sounds();

	//Unload Sprites
	unload_game_sprites();

	//Unload Levels
	unload_levels();
}