//Source file for Level functions.

#include <level.h>
#include <attr.h>
#include <log.h>
#include <util.h>
#include <gameglobal.h>

//Load ASCII Art from Level file
aamap *load_asciimap_level_file(FILE *flevel, char *levelname){

	aamap *newaam=NULL;
	char filename[MAXSTR],aafilename[MAXSTR];
	FILE *fascii=NULL;

	if(!levelname||!*levelname) return newaam;

	*filename=0;
	fscanf(flevel,"%s",filename);

	if(!filename||!*filename) return newaam;
	
	*aafilename=0;				
	sprintf(aafilename,"%s%c%s%c%s%c%s","Data",PATHSEP,"Levels",PATHSEP,levelname,PATHSEP,filename);

	fascii=fopen(aafilename,"r");
	if(!fascii) return newaam;

	//Loading ASCIIMap
	#ifdef VERBOSE
		log_printf("Loading ASCIIMap from %s",aafilename);
	#endif //VERBOSE

	newaam=load_asciimap_file(fascii);
	
	fclose(fascii);	fascii=NULL;
			
	return newaam;
}

//Load AttributeMap from Level File
attrmap *load_attrmap_level_file(FILE *flevel, char *levelname){
	attrmap *newam=NULL;
	char filename[MAXSTR],attrfilename[MAXSTR];
	FILE *fattr=NULL;

	if(!levelname||!*levelname) return newam;

	*filename=0;
	fscanf(flevel,"%s",filename);

	if(!filename||!*filename) return newam;
	
	*attrfilename=0;				
	sprintf(attrfilename,"%s%c%s%c%s%c%s","Data",PATHSEP,"Levels",PATHSEP,levelname,PATHSEP,filename);

	fattr=fopen(attrfilename,"r");
	if(!fattr) return newam;

	//Loading AttributeMap
	#ifdef VERBOSE
		log_printf("Loading AttributeMap from %s",attrfilename);
	#endif //VERBOSE

	newam=load_attrmap_file(fattr);
	
	fclose(fattr);	fattr=NULL;
			
	return newam;
}

//Unload Level
void unload_level(level *givlevel){
	int i;
	if(givlevel){
		for(i=0;i<givlevel->nlayers;++i){
			if(givlevel->layertypes[i]==LEVEL_ATTRMAP) {
				unload_attrmap(givlevel->layers[i]);
				givlevel->layers[i]=NULL;
				givlevel->layertypes[i]=LEVEL_NULL;
			} 
			else if(givlevel->layertypes[i]==LEVEL_AARTMAP) {
				unload_asciimap(givlevel->layers[i]);
				givlevel->layers[i]=NULL;
				givlevel->layertypes[i]=LEVEL_NULL;
			} 
			else ;
		}	
		free(givlevel->layertypes);
		givlevel->layertypes=NULL;
		free(givlevel->layers);
		givlevel->layers=NULL;
		
		givlevel->nlayers=0;
		givlevel->am=NULL;
		//Destroying Level
		free(givlevel);
		#ifdef VERBOSE
			log_printf("Destroyed Level");
		#endif //VERBOSE
	}
}

//Create Level
level *create_level(int nlayers){
	int i;
	level *newlevel=NULL;	
	newlevel=malloc(sizeof(level));	
	if(newlevel) {
		newlevel->nlayers=0;	
		if(nlayers>0) {
			#ifdef VERBOSE
				log_printf("Created %d layers",nlayers);
			#endif //VERBOSE
			newlevel->nlayers=nlayers;	
			#ifdef VERBOSE
				log_printf("PreInitializing Layer Types");
			#endif //VERBOSE
			newlevel->layertypes=malloc(sizeof(int)*nlayers);	
			if(newlevel->layertypes) {
				for(i=0;i<newlevel->nlayers;++i)
					newlevel->layertypes[i]=LEVEL_NULL;
			}
			#ifdef VERBOSE
				log_printf("PreInitializing Layer Data");
			#endif //VERBOSE
			newlevel->layers=malloc(sizeof(void *)*nlayers);	
			if(newlevel->layers) {
				for(i=0;i<newlevel->nlayers;++i)
					newlevel->layers[i]=NULL;
			}	
		}	
		if(!newlevel->nlayers||!newlevel->layertypes||!newlevel->layers)  {
			unload_level(newlevel);
			newlevel=NULL;
		}
	}
	return newlevel;
}

//Load Level from a pre opened file
level *load_level_file(FILE *flevel, char *levelname){
	int i;
	int nlayers;
	char layertype[MAXSTR];
	level *newlevel=NULL;

	nlayers=0;
	fscanf(flevel,"%d",&nlayers);	
		
	newlevel=create_level(nlayers);
	if(newlevel) {
		for(i=0;i<newlevel->nlayers;++i) {
			fscanf(flevel,"%s",layertype);
			if(!strcasecmp("attr",layertype)) {
				//Attributes
				#ifdef VERBOSE
					log_printf("Layer %d: %s",i,"Attributes");
				#endif //VERBOSE
				newlevel->layers[i]=load_attrmap_level_file(flevel,levelname);
				newlevel->layertypes[i]=LEVEL_ATTRMAP;
				newlevel->am=(attrmap *)newlevel->layers[i];
			}
			else if(!strcasecmp("ascii",layertype)) {
				//Ascii ART
				#ifdef VERBOSE
					log_printf("Layer %d: %s",i,"ASCII Art");
				#endif //VERBOSE
				newlevel->layers[i]=load_asciimap_level_file(flevel,levelname);
				newlevel->layertypes[i]=LEVEL_AARTMAP;
		
			}			
		}
	}	

	return newlevel;
}

//Load Level from a file, name is identified by 'levelname'
level *load_level(char *levelname){
	char levelfilename[MAXSTR];
	FILE *flevel;
	level *newlevel=NULL;

	if(!levelname||!*levelname) return newlevel;

	*levelfilename=0;
	//From: Data/Levels/levelname/level.txt
	sprintf(levelfilename,"%s%c%s%c%s%c%s","Data",PATHSEP,"Levels",PATHSEP,levelname,PATHSEP,"level.txt");

	flevel=fopen(levelfilename,"r");
	if(!flevel) return newlevel;	

	//Loading Level
	#ifdef VERBOSE
		log_printf("Loading Level from: %s",levelfilename);
	#endif //VERBOSE
		
	newlevel=load_level_file(flevel,levelname);

	fclose(flevel);	
	return newlevel;
}


//Unload all Levels
void unload_levels(){
	int i;
	if(levels) {
		//Unloading Levels
		#ifdef VERBOSE
			log_printf("Destroying Levels");
		#endif //VERBOSE
		for(i=0;i<nlevels;++i){
			//Unload Each level
			unload_level(levels[i]);
			levels[i]=NULL;
		}
		free(levels);
		levels=NULL;
	}
	nlevels=0;
	curlevel=-1;
}


//Load all Levels
int load_levels(){
	int i;
	int status=-1;
	char filename[MAXSTR],levelname[MAXSTR];
	FILE *flevels;
	unload_levels();

	//From: Data/levels.txt
	sprintf(filename,"%s%c%s","Data",PATHSEP,"levels.txt");

	flevels=fopen(filename,"r");
	if(!flevels) return status;

	//Loading Level
	#ifdef VERBOSE
		log_printf("Loading Levels from list: %s",filename);
	#endif //VERBOSE

	nlevels=0; fscanf(flevels,"%d",&nlevels);	
	levels=malloc(sizeof(level *)*nlevels);
	if(levels) {
		#ifdef VERBOSE
			log_printf("Loading %d Levels",nlevels);
		#endif //VERBOSE
		for(i=0;i<nlevels;++i){
			*levelname=0;
			fscanf(flevels,"%s",levelname);
			if(*levelname) {
				#ifdef VERBOSE
					log_printf("Loading Level %d: %s",i,levelname);
				#endif //VERBOSE
				levels[i]=load_level(strip_crlfs(levelname));			
				curlevel=i;
			}
		}				
	}	
	fclose(flevels);
	return 0;
}