//Source file for Menu.

#include <header.h>
#include <game.h>
#include <gamestate.h>
#include <gameglobal.h>
#include <global.h>
#include <ascii.h>
#include <ascrender.h>

//Menu Items
#define MENU_NULL -1
#define MENU_GAME 0
#define MENU_HELP 1
#define MENU_EXIT 2

//Simple Menu
int menu_selected=MENU_NULL;
int menu_x=0;

//ASCII Sprites 
aasprite *aatitle=NULL,*aatitbg=NULL;

//Game Menu Initialization
void game_menu_init(){
	int i=0,j=0,w=0,h=0;
	unsigned char *data=NULL;
	char aafilename[MAXSTR],filename[MAXSTR],typename[MAXSTR];
	FILE *fascii=NULL;

	strcpy(typename,"Game");

	strcpy(filename,"title.txt");
	*aafilename=0;				
	sprintf(aafilename,"%s%c%s%c%s%c%s","Data",PATHSEP,"Sprites",PATHSEP,typename,PATHSEP,filename);
	fascii=fopen(aafilename,"r"); 
	if(fascii) {	
		//Loading ASCIIMap
		#ifdef VERBOSE
			log_printf("Loading ASCIIMap from %s",aafilename);
		#endif //VERBOSE
		aatitle=load_asciiart_file(fascii); fclose(fascii); fascii=NULL;
	}

	w=dbuffer->w/xmul; if(dbuffer->w%xmul) ++w;
	h=dbuffer->h/ymul; if(dbuffer->h%ymul) ++h;
	aatitbg=create_asciiart(w,h);

	//The Funky Cross-Pattern :)
	data=aatitbg->data;	
	for(i=0;i<aatitbg->height;++i) {
		for(j=0;j<aatitbg->width;++j) {
			if((i+j)%2) *data='\\';
			else *data='/';
			data++;
		}
	}
	aatitbg->r=191; aatitbg->g=191; aatitbg->b=191; aatitbg->a=255;

	menu_selected=MENU_GAME;
	menu_x=0;
}

//Menu Logic, Termination
void game_menu_logic(){
	menu_x++; //menu_x+=xmul has a dizzying effect
	if(menu_x>=dbuffer->w) menu_x=0;
}

//Game Menu Input and State Dispatch
void game_menu_input(){
	int k=-1;
	if(!keypressed()) return;

	k=readkey(); k>>=8;

	if(k==KEY_ENTER) {
		if(menu_selected==MENU_GAME) 
			state_switch(GAME_PLAY);
		else if(menu_selected==MENU_HELP) 
			state_switch(GAME_HELP);
		else if(menu_selected==MENU_EXIT) 
			{ terminate_state_loop(); return; }
	}
		
	if(k==KEY_UP) menu_selected--;
	if(k==KEY_DOWN) menu_selected++;

	if(menu_selected<MENU_GAME) menu_selected=MENU_GAME;
	if(menu_selected>MENU_EXIT) menu_selected=MENU_EXIT;
}

//Game Menu Rendering
void game_menu_render(){

	//BackGround
	clear_to_color(dbuffer,makecol(220,220,220));

	//Title
	draw_asciiart(dbuffer, aatitbg, menu_x, 0 );
	draw_asciiart(dbuffer, aatitbg, menu_x-(aatitbg->width*xmul), 0 );

	//Title
	draw_asciiart(dbuffer, aatitle, (dbuffer->w-(aatitle->width*xmul))/2, 30);

	//Game
	if(menu_selected==MENU_GAME) 
		textout_centre_ex(dbuffer, font, "Game", dbuffer->w/2, 150, makecol(255,0,0), -1);
	else
		textout_centre_ex(dbuffer, font, "Game", dbuffer->w/2, 150, makecol(64,0,0), -1);

	//Help
	if(menu_selected==MENU_HELP) 
		textout_centre_ex(dbuffer, font, "Help", dbuffer->w/2, 170, makecol(255,0,0), -1);
	else
		textout_centre_ex(dbuffer, font, "Help", dbuffer->w/2, 170, makecol(64,0,0), -1);

	//Exit
	if(menu_selected==MENU_EXIT) 
		textout_centre_ex(dbuffer, font, "Exit", dbuffer->w/2, 190, makecol(255,0,0), -1);
	else
		textout_centre_ex(dbuffer, font, "Exit", dbuffer->w/2, 190, makecol(64,0,0), -1);

}

//Game Menu Uninitialize
void game_menu_quit(){
	if(aatitle) unload_asciiart(aatitle);
	aatitle=NULL;	

	if(aatitbg) unload_asciiart(aatitbg);
	aatitbg=NULL;	
	
	menu_selected=MENU_NULL;
	
	menu_x=0;
}